@@ -28,7 +28,7 @@ except NameError:
     basestring = str
 
 
-class Grader():
+class Grader:
 
     ERROR_REPLY = u'服务器通信错误。'.encode('utf-8')
     TIMEOUT_REPLY = u'请让我再想想。'.encode('utf-8')
@@ -57,6 +57,8 @@ class Grader():
         self.sessions = {}
         self.answers = {}
 
+        self.uid_prefix = 'Console.Test.'
+
     def test_robot(self, name, questions, answers, sessions=None):
         if sessions is None:
             sessions = {}
@@ -66,7 +68,7 @@ class Grader():
         else:
             test_service = pMsgHandler(self.config, test_logger)
 
-        uid = "grader_" + id_generator(20)
+        uid = self.uid_prefix + id_generator(20)
         last_session = 1
 
         test_service.robot_name = name
@@ -78,7 +80,7 @@ class Grader():
         total_time = 0
         for i, question in questions.items():
             if sessions.get(i, -1) != last_session:
-                uid = "grader_" + id_generator(20)
+                uid = self.uid_prefix + id_generator(20)
                 last_session = sessions.get(i, -1)
 
             process_time = time.time()
@@ -357,6 +359,8 @@ class Grader():
         self.print_correct_answer = configuration.get('print_correct_answer', self.print_correct_answer)
         self.print_csv = configuration.get('print_csv', self.print_csv)
 
+        self.uid_prefix = config.get_config('uid_prefix', self.uid_prefix)
+
         if self.questions and len(self.questions) > 0:
             report_logger.info("grader ready, there are {0} questions".format(len(self.questions)))
 
@@ -398,7 +402,8 @@ class Grader():
             if data_url:
                 r = requests.get(
                     data_url,
-                    headers={'key': data_token}
+                    headers={'key': data_token},
+                    timeout=2
                 )
                 extra_data = r.json()
             else:
